'use strict';

var _ = require('lodash'),
		Context = require('./context/Context'),
		EventEmitter = require('events'),
		QueryBuilder = require('./query/QueryBuilder'),
		RepositoryConfig = require('./RepositoryConfig');

class Repository extends EventEmitter {
	constructor(config) {
		super();

		if(config instanceof RepositoryConfig === false) {
			throw new Error('Invalid config');
		}

		if(!config.dataProvider) {
			throw new Error('You must specify a data provider');
		}

		this.contexts = {};
		this.config = config;
		this.dataProvider = this.config.dataProvider;
		this.name = this.config.name;
	}

	hasContext (name) {
		return this.contexts[name] instanceof Context;
	}

	createContext (name) {
		if(!this.hasContext(name)) {
			var context = new Context(name);

			// using updateContext.bind to generate a handler is harder to test
			// keep calling with the closure's "self" reference
			context.on('update', () => {
				this.updateContext(context);
			});

			this.contexts[name] = context;
		}

		return this.getContext(name);
	}

	getContext (name) {
		return this.contexts[name];
	}

	updateContext (context) {
		var state = context.toJSON();

		this.dataProvider.find(this.name, state).then(function (data) {
			context.setData(data);
		}, function (err) {
			context.setError(err);
		});

		return this;
	}

	removeContext (name) {
		delete this.contexts[name];

		return this;
	}

	find (queryBuilder) {
		if(queryBuilder instanceof QueryBuilder === false) {
			throw new Error('Invalid query builder');
		}

		var params = queryBuilder.toJSON();

		return this.dataProvider.find(this.name, params);
	}

	findOne (id) {
		return this.dataProvider.findOne(this.name, id);
	}

	removeOne (id) {
		return this.dataProvider.removeOne(this.name, id);
	}

	remove (ids) {
		return this.dataProvider.remove(this.name, ids);
	}

	saveOne (entity) {
    var self = this;

		return this.dataProvider.saveOne(this.name, entity).then(function (response) {
			self.emit(self.EVENTS.UPDATE);

			return response;
		});
	}

	save (entities) {
		if(entities.length === 0) {
			return this.dataProvider.error(this.ERRORS.EMPTY_ENTITY_SET);
		}

		var self = this;

		return this.dataProvider.save(this.name, entities).then(function (response) {
			self.emit(self.EVENTS.UPDATE);

			return response;
		});
	}
}

_.extend(Repository.prototype, {
	EVENTS: {
		UPDATE: 'update'
	},
	ERRORS: {
		EMPTY_ENTITY_SET: 'EMPTY_ENTITY_SET',
		INVALID_ENTITY_SET: 'INVALID_ENTITY_SET'
	}
});

module.exports = Repository;
