'use strict';

var _ = require('lodash'),
		EventEmitter = require('events');

class QueryPagination extends EventEmitter {
	constructor() {
		super();
		this.reset();
	}

	isValidPage(page) {
		return page > 0 && page <= this.totalPages;
	}

	next() {
		if(!this.isValidPage(this.currentPage + 1)) {
			return this;
		}

		this.setPage(this.currentPage + 1);

		return this;
	}

	previous() {
		if(!this.isValidPage(this.currentPage - 1)) {
			return this;
		}

		this.setPage(this.currentPage - 1);

		return this;
	}

	setPage(page) {
		var oldPage = this.currentPage;

		this.currentPage = page;

		this.emit('update');

		return this;
	}

	setItemsPerPage(itemsPerPage) {
		this.setState({ itemsPerPage: itemsPerPage });
		this.refresh();

		return this;
	}

	setState(config) {
		var totalItems = config.totalItems;

		if(!(totalItems === 0) && !_.isUndefined(totalItems) && isNaN(totalItems)) {
			throw new Error('Invalid total items property');
		}

		_.extend(this, config);
		this.refresh();

		return this;
	}

	refresh() {
		if(this.currentPage < 1 || !this.isValidPage(this.currentPage)) {
			this.currentPage = this.defaults.currentPage;
		}

		if(this.itemsPerPage < 1) {
			this.itemsPerPage = Math.floor(this.defaults.itemsPerPage);
		}

		this.totalItems = this.totalItems || 0;
		this.itemsPerPage = parseInt(this.itemsPerPage);

		this.totalPages = this.totalItems / this.itemsPerPage;

		if(!this.totalPages || _.isNumber(this.totalPages)) {
			this.totalPages = Math.round(this.totalItems / this.itemsPerPage);
		}

		this._pages = [];

		for(var i=0; i<this.totalPages; i++) {
			this._pages.push(i + 1);
		}
	}

	reset() {
		this.setState({
			currentPage: 0,
			itemsPerPage: 0,
			totalItems: 0,
			totalPages: 0
		});

		return this;
	}

	last() {
		if(!this.isValidPage(this.totalPages)) {
			return this;
		}

		this.setPage(this.totalPages);

		return this;
	}

	first() {
		if(!this.currentPage) {
			return this;
		}

		return this.setPage(1);
	}

	toJSON () {
		var state = {};
		var keys = ['itemsPerPage', 'currentPage', 'count'];

		_.forEach(keys, key => {
			if(!_.isUndefined(this[key])) {
				state[key] = this[key];
			}
		});

		return state;
	}
}

QueryPagination.prototype.defaults = {
	itemsPerPage: 4,
	currentPage: 1,
	totalItems: 0
};

module.exports = QueryPagination;
